package com.goafanti.achievement.controller;

import java.util.Arrays;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import org.springframework.beans.BeanUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import com.goafanti.achievement.bo.InputAchievement;
import com.goafanti.achievement.service.AchievementService;
import com.goafanti.admin.service.AttachmentService;
import com.goafanti.common.bo.Result;
import com.goafanti.common.constant.AFTConstants;
import com.goafanti.common.constant.ErrorConstants;
import com.goafanti.common.controller.CertifyApiController;
import com.goafanti.common.enums.AchievementAuditStatus;
import com.goafanti.common.enums.AchievementFields;
import com.goafanti.common.enums.AttachmentType;
import com.goafanti.common.model.Achievement;
import com.goafanti.common.model.Attachment;
import com.goafanti.common.utils.StringUtils;
import com.goafanti.core.shiro.token.TokenManager;

@RestController
@RequestMapping(value = "/api/user/achievement")
public class UserAchievementApiController extends CertifyApiController {
	@Resource
	private AchievementService		achievementService;
	@Resource
	private AttachmentService			aftFileService;
	

	/**
	 * 成果需求匹配列表
	 */
	@RequestMapping(value = "/achievementDemand", method = RequestMethod.GET)
	public Result achievementDemand(String id) {
		Result res = new Result();
		res.setData(achievementService.selectAchievementDemandListByAchievementId(id));
		return res;
	}

	/**
	 * 下载科技成果批量导入Excel模板
	 */
	@RequestMapping(value = "/downloadTemplate", method = RequestMethod.GET)
	public Result downloadTemplateFile(HttpServletResponse response, String sign) {
		Result res = new Result();
		AttachmentType attachmentType = AttachmentType.getField(sign);
		if (attachmentType == AttachmentType.ACHIEVEMENT_TEMPLATE) {
			String fileName = "";
			Attachment af = aftFileService.selectAftFileBySign(sign);
			if (null == af) {
				res.getError().add(buildError(ErrorConstants.FILE_NON_EXISTENT, "", "找不到文件！"));
			} else {
				String path = af.getFilePath();
				String suffix = path.substring(path.lastIndexOf("."));
				fileName = AttachmentType.ACHIEVEMENT_TEMPLATE.getDesc() + suffix;
				downloadFile(response, fileName, path);
			}
		} else {
			res.getError().add(buildError(ErrorConstants.PARAM_ERROR, "", "附件标示"));
		}
		return res;
	}

	/**
	 * 成果撤消发布(下架)
	 */
	@RequestMapping(value = "/offShelf", method = RequestMethod.POST)
	public Result offShelf(String id) {
		Result res = new Result();
		if (StringUtils.isBlank(id)) {
			res.getError().add(buildError(ErrorConstants.PARAM_EMPTY_ERROR, "找不到成果ID", "成果ID"));
			return res;
		}

		Achievement a = achievementService.selectByPrimaryKey(id);

		if (null == a) {
			res.getError().add(buildError(ErrorConstants.PARAM_ERROR, "", "需求ID"));
			return res;
		}

		res.setData(achievementService.updateReleaseStatus(a));
		return res;
	}

	/**
	 * 成果列表
	 */
	@RequestMapping(value = "/list", method = RequestMethod.GET)
	private Result list(Integer auditStatus, Integer serialNumber, String name, String keyword, Integer category,
			Integer ownerType, Integer status, String releaseDateStartDate, String releaseDateEndDate,String createDateStartDate, String createDateEndDate,
			Integer releaseStatus, String pageNo, String pageSize) {
		Result res = new Result();
		Integer pNo = 1;
		Integer pSize = 10;
		if (StringUtils.isNumeric(pageSize)) {
			pSize = Integer.parseInt(pageSize);
		}

		if (StringUtils.isNumeric(pageNo)) {
			pNo = Integer.parseInt(pageNo);
		}
		res.setData(achievementService.listUserAchievement(auditStatus, serialNumber, name, keyword, category,
				ownerType, status, releaseDateStartDate, releaseDateEndDate, createDateStartDate,  createDateEndDate, releaseStatus, pNo, pSize));
		return res;
	}

	/**
	 * 新增成果
	 */
	@RequestMapping(value = "/apply", method = RequestMethod.POST)
	private Result apply(@Valid InputAchievement ia, BindingResult bindingResult,
			@RequestParam(name = "keywords[]", required = false) String[] keywords) {
		Result res = new Result();

		if (bindingResult.hasErrors()) {
			res.getError().add(buildErrorByMsg(bindingResult.getFieldError().getDefaultMessage(),
					AchievementFields.getFieldDesc(bindingResult.getFieldError().getField())));
			return res;
		}

		res = disposeInputAchievement(res, ia, keywords);
		if (!res.getError().isEmpty()) {
			return res;
		}

		Achievement a = new Achievement();
		BeanUtils.copyProperties(ia, a);
		a.setOwnerId(TokenManager.getUserId());
		achievementService.saveAchievement(a, keywords);
		return res;
	}

	/**
	 * 个人成果详情详情
	 */
	@RequestMapping(value = "/userDetail", method = RequestMethod.GET)
	private Result userDetail(String id) {
		Result res = new Result();
		if (StringUtils.isBlank(id)) {
			res.getError().add(buildError(ErrorConstants.PARAM_EMPTY_ERROR, "找不到成果ID", "成果ID"));
			return res;
		}
		res.setData(achievementService.selectUserOwnerDetail(id));
		return res;
	}

	/**
	 * 组织用户成果详情详情
	 */
	@RequestMapping(value = "/orgDetail", method = RequestMethod.GET)
	private Result orgDetail(String id) {
		Result res = new Result();
		if (StringUtils.isBlank(id)) {
			res.getError().add(buildError(ErrorConstants.PARAM_EMPTY_ERROR, "找不到成果ID", "成果ID"));
			return res;
		}
		res.setData(achievementService.selectOrgOwnerDetail(id));
		return res;
	}

	/**
	 * 修改成果
	 */
	@RequestMapping(value = "/update", method = RequestMethod.POST)
	public Result update(@Valid InputAchievement ia, BindingResult bindingResult,
			@RequestParam(name = "keywords[]", required = false) String[] keywords) {
		Result res = new Result();
		if (bindingResult.hasErrors()) {
			res.getError().add(buildErrorByMsg(bindingResult.getFieldError().getDefaultMessage(),
					AchievementFields.getFieldDesc(bindingResult.getFieldError().getField())));
			return res;
		}

		if (StringUtils.isBlank(ia.getId())) {
			res.getError().add(buildError(ErrorConstants.PARAM_EMPTY_ERROR, "找不到成果ID", "成果ID"));
			return res;
		}

		if (!AchievementAuditStatus.CREATE.getCode().equals(ia.getAuditStatus())
				&& !AchievementAuditStatus.SUBMIT.getCode().equals(ia.getAuditStatus())
				&& !AchievementAuditStatus.UNAUDITED.getCode().equals(ia.getAuditStatus())) {
			res.getError().add(buildError(ErrorConstants.PARAM_ERROR, "", "当前状态无法提交审核！"));
			return res;
		}

		res = disposeInputAchievement(res, ia, keywords);
		if (!res.getError().isEmpty()) {
			return res;
		}

		Achievement a = new Achievement();
		BeanUtils.copyProperties(ia, a);
		achievementService.updateAchievement(a, keywords, null);
		return res;
	}

	/**
	 * 下载文本文件
	 */
	@RequestMapping(value = "/download", method = RequestMethod.GET)
	public Result download(HttpServletResponse response, String id, String sign) {
		Result res = new Result();

		if (StringUtils.isBlank(id)) {
			res.getError().add(buildError(ErrorConstants.PARAM_ERROR, "", "科技成果ID"));
			return res;
		}

		Achievement a = achievementService.selectByPrimaryKey(id);
		if (null == a) {
			res.getError().add(buildError(ErrorConstants.PARAM_ERROR, "", "科技成果ID"));
			return res;
		}

		AttachmentType attachmentType = AttachmentType.getField(sign);
		if (attachmentType == AttachmentType.ACHIEVEMENT_MATURITY_TEXT_FILE) {
			downloadUnPrivateFile(response, a.getMaturityTextFileDownloadFileName(), a.getMaturityTextFileUrl());
		} else if (attachmentType == AttachmentType.ACHIEVEMENT_TECH_PLAN) {
			downloadUnPrivateFile(response, a.getTechPlanDownloadFileName(), a.getTechPlanUrl());
		} else if (attachmentType == AttachmentType.ACHIEVEMENT_BUSINESS_PLAN) {
			downloadUnPrivateFile(response, a.getBusinessPlanDownloadFileName(), a.getBusinessPlanUrl());
		} else {
			res.getError().add(buildError(ErrorConstants.PARAM_ERROR, "", "附件标示"));
		}

		return res;
	}

	/**
	 * 文本文件上传
	 */
	@RequestMapping(value = "/uploadTextFile", method = RequestMethod.POST)
	public Result uploadTextFile(HttpServletRequest req, String sign) {
		Result res = new Result();

		AttachmentType attachmentType = AttachmentType.getField(sign);

		if (attachmentType == AttachmentType.ACHIEVEMENT_MATURITY_TEXT_FILE
				|| attachmentType == AttachmentType.ACHIEVEMENT_TECH_PLAN
				|| attachmentType == AttachmentType.ACHIEVEMENT_BUSINESS_PLAN) {
			res.setData(handleFiles(res, "/achievement/", false, req, sign, "achievement"));
		} else {
			res.getError().add(buildError(ErrorConstants.PARAM_ERROR, "", "附件标示"));
		}
		return res;
	}

	/**
	 * 图片上传
	 */
	@RequestMapping(value = "/uploadPicture", method = RequestMethod.POST)
	public Result uploadPicture(HttpServletRequest req, String sign) {
		Result res = new Result();

		AttachmentType attachmentType = AttachmentType.getField(sign);

		if (attachmentType == AttachmentType.ACHIEVEMENT_TECHNICAL_PICTURE
				|| attachmentType == AttachmentType.ACHIEVEMENT_MATURITY_PICTURE) {
			res.setData(handleFiles(res, "/achievement/", false, req, sign, "achievement"));
		} else {
			res.getError().add(buildError(ErrorConstants.PARAM_ERROR, "", "附件标示"));
		}

		return res;
	}

	/**
	 * 删除
	 */
	@RequestMapping(value = "/delete", method = RequestMethod.POST)
	private Result delete(@RequestParam(name = "ids[]", required = false) String[] ids) {
		Result res = new Result();
		if (ids == null || ids.length != 1) {
			res.getError().add(buildError(ErrorConstants.PARAM_ERROR, "", ""));
		} else {
			
			res.setData(achievementService.deleteByPrimaryKey(Arrays.asList(ids)));
		}
		return res;
	}

	private Result disposeInputAchievement(Result res, InputAchievement ia, String[] keywords) {
		if (StringUtils.isBlank(ia.getName())) {
			res.getError().add(buildError(ErrorConstants.PARAM_EMPTY_ERROR, "找不到成果名称", "成果名称"));
			return res;
		}

		if (null == ia.getCategory()) {
			res.getError().add(buildError(ErrorConstants.PARAM_EMPTY_ERROR, "找不到数据类别", "数据类别"));
			return res;
		}

		if (StringUtils.isBlank(ia.getKeyword())) {
			res.getError().add(buildError(ErrorConstants.PARAM_EMPTY_ERROR, "找不到关键词", "关键词"));
			return res;
		}

		if (null == keywords || keywords.length < 1) {
			res.getError().add(buildError(ErrorConstants.PARAM_EMPTY_ERROR, "找不到关键词", "关键词"));
			return res;
		}

		if (null == ia.getOwnerType()) {
			res.getError().add(buildError(ErrorConstants.PARAM_EMPTY_ERROR, "找不到所有人类型", "所有人类型"));
			return res;
		}

		if (null == ia.getCooperationMode()) {
			res.getError().add(buildError(ErrorConstants.PARAM_EMPTY_ERROR, "找不到合作方式", "合作方式"));
			return res;
		}

		for (int i = 0; i < keywords.length; i++) {
			if (AFTConstants.KEYWORDLENTH < keywords[i].length()) {
				res.getError().add(buildError(ErrorConstants.PARAM_ERROR, "", "关键词长度"));
				return res;
			}
		}
		return res;
	}

}
